package nz.co.cloudstore.airconsoledemo.ui;

import android.content.Context;
import android.content.Intent;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import nz.co.cloudstore.airconsole.AirconsoleDevice;
import nz.co.cloudstore.airconsoledemo.DemoAppGlobals;
import nz.co.cloudstore.airconsoledemo.R;
import nz.co.cloudstore.airconsoledemo.adapters.ButtonViewHolder;
import nz.co.cloudstore.airconsoledemo.adapters.ItemViewHolder;
import nz.co.cloudstore.airconsoledemo.adapters.TitleViewHolder;

public class DeviceActivity extends AppCompatActivity {

    private static final String TAG = "DeviceActivity";

    private static final int TYPE_TITLE = 0;
    private static final int TYPE_ITEM = 1;
    private static final int TYPE_OPTION = 2;
    private static final int TYPE_BUTTON = 3;

    private RecyclerView recyclerView;
    private DeviceAdapter adapter;
    private AirconsoleDevice device;

    private RecyclerViewOnClickListener mListener;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_device);
        recyclerView = findViewById(R.id.deviceDetailsRecyclerView);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));
        device = DemoAppGlobals.getDevice();



        adapter = new DeviceAdapter(device, this);
        recyclerView.setAdapter(adapter);

        mListener = new RecyclerViewOnClickListener() {
            @Override
            public void onClick(View view, int position) {

            }
        };
    }

    private class DeviceAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

        private AirconsoleDevice device;
        private Context context;

        DeviceAdapter(AirconsoleDevice device, Context context) {
            this.device = device;
            this.context = context;
        }

        @Override
        public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            if (viewType == TYPE_TITLE) {
                return new TitleViewHolder(LayoutInflater.from(context).inflate(R.layout.list_item_title, parent, false));
            } else if (viewType == TYPE_ITEM) {
                return new ItemViewHolder(LayoutInflater.from(context).inflate(R.layout.list_item_value, parent, false), mListener);
            } else if (viewType == TYPE_BUTTON) {
                return new ButtonViewHolder(LayoutInflater.from(context).inflate(R.layout.list_item_button, parent, false));
            }
            return null;
        }

        @Override
        public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
            if (holder instanceof TitleViewHolder) {
                TitleViewHolder titleViewHolder = (TitleViewHolder) holder;
                switch (position) {
                    case 0:
                        titleViewHolder.title.setText(getString(R.string.device_details));
                        break;
                }
            } else if (holder instanceof ItemViewHolder) {
                ItemViewHolder itemViewHolder = (ItemViewHolder) holder;
                switch (position) {
                    case 1: {
                        itemViewHolder.title.setText(getString(R.string.device_name));
                        itemViewHolder.value.setText(device.getName());
                        break;
                    }
                    case 2: {
                        itemViewHolder.title.setText(getString(R.string.device_address));
                        itemViewHolder.value.setText(device.address());
                        break;
                    }
                    case 3: {
                        itemViewHolder.title.setText(getString(R.string.device_type));
                        itemViewHolder.value.setText(device.getDeviceType());
                        break;
                    }
                    case 4: {
                        itemViewHolder.title.setText(getString(R.string.device_physical_ports));
                        itemViewHolder.value.setText(String.valueOf(device.getPortCount()));
                        break;
                    }
                    case 5: {
                        itemViewHolder.title.setText(getString(R.string.device_firmware_version));
                        itemViewHolder.value.setText(device.getFirmwareVersion());
                        break;
                    }
                }
            } else if (holder instanceof ButtonViewHolder) {
                ButtonViewHolder buttonViewHolder = (ButtonViewHolder) holder;
                switch (position) {
                    case 6:
                        buttonViewHolder.button.setText(getString(R.string.device_connect));
                        buttonViewHolder.button.setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View view) {
                                Intent intent = new Intent(DeviceActivity.this, SessionActivity.class);
                                startActivity(intent);
                            }
                        });
                }
            }
        }

        @Override
        public int getItemCount() {
            return 7;
        }

        @Override
        public int getItemViewType(int position) {

            int itemType = -1;

            switch (position) {
                case 0:
                    itemType = TYPE_TITLE;
                    break;
                case 1:
                case 2:
                case 3:
                case 4:
                case 5:
                    itemType = TYPE_ITEM;
                    break;
                case 6:
                    itemType = TYPE_BUTTON;
                    break;
            }

            return itemType;
        }
    }
}
