package nz.co.cloudstore.airconsoledemo.ui;

import android.Manifest;
import android.bluetooth.BluetoothAdapter;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.os.Handler;
import android.os.Looper;
import android.os.Bundle;
import android.support.design.widget.Snackbar;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.SearchView;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import nz.co.cloudstore.airconsole.Airconsole;
import nz.co.cloudstore.airconsole.AirconsoleDevice;
import nz.co.cloudstore.airconsole.AirconsoleMgr;
import nz.co.cloudstore.airconsole.AirconsoleMgrListener;
import nz.co.cloudstore.airconsoledemo.DemoAppGlobals;
import nz.co.cloudstore.airconsoledemo.adapters.ItemViewHolder;
import nz.co.cloudstore.airconsoledemo.data.PrefsStore;
import nz.co.cloudstore.airconsoledemo.R;

public class DiscoveryActivity extends AppCompatActivity implements AirconsoleMgrListener, SearchView.OnQueryTextListener {

    // Change these booleans to indicate what transports you want to scan for Airconsoles on
    private static boolean AIRCONSOLE_WIFI = true;
    private static boolean AIRCONSOLE_BT   = true;
    private static boolean AIRCONSOLE_BTLE = true;

    private static final String TAG = "NewDiscoveryActivity";

    private AirconsoleMgr mgr = Airconsole.getAirconsoleMgr();

    private WifiManager.MulticastLock multicastLock = null;

    private Handler updateHandler;
    private LayoutInflater inflater;

    private RecyclerView recyclerView;
    private RecyclerView.LayoutManager layoutManager;

    private RecyclerViewOnClickListener mListener;
    private TextView versionFooter;
    private TextView detectedDevicesFooter;
    private LinearLayout infoLinearLayout;

    private List<AirconsoleDevice> devices;
    private DiscoveryAdapter adapter;

    private boolean filteringSearchResults;

    private final int REQUEST_ENABLE_BT = 6060;
    private final int REQUEST_ENABLE_FINE_LOCATION = 5050;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        inflater = (LayoutInflater)getSystemService(Context.LAYOUT_INFLATER_SERVICE);

        setContentView(R.layout.activity_discovery);

        updateHandler = new Handler(Looper.getMainLooper());

        recyclerView = findViewById(R.id.deviceDetailsRecyclerView);
        layoutManager = new LinearLayoutManager(this);
        recyclerView.setLayoutManager(layoutManager);

        detectedDevicesFooter = findViewById(R.id.detectedDevicesTextView);
        versionFooter = findViewById(R.id.sdkVersionTextView);
        versionFooter.setText(getString(R.string.version_footer, Airconsole.getSDKVersion()));
        infoLinearLayout = findViewById(R.id.infoLinearLayout);

        Log.d(TAG, "About to initialise manager");

        mgr.addAirconsoleMgrListener(this);

        loadStoredPrefs();

        updateDeviceList();

        mListener = new RecyclerViewOnClickListener() {
            @Override
            public void onClick(View view, int position) {
                AirconsoleDevice device = devices.get(position);
                Intent intent = new Intent(DiscoveryActivity.this, DeviceActivity.class);
                DemoAppGlobals.setDevice(device);
                startActivity(intent);
            }
        };
        if (!isBluetoothEnabled()) {
            displayEnabledBluetoothPrompt();
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        airconsoleScan(true);
    }

    @Override
    public void onPause() {
        super.onPause();
        airconsoleScan(false);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.discovery_menu, menu);
        final MenuItem searchItem = menu.findItem(R.id.action_search);
        final SearchView searchView = (SearchView) searchItem.getActionView();
        searchView.setOnQueryTextListener(this);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        for (int i = 0; i < menu.size(); i++) {
            switch (menu.getItem(i).getItemId()) {
                case R.id.wifiItem: {
                    menu.getItem(i).setChecked(PrefsStore.getInstance(this).getIpTransportEnabled());
                    break;
                }
                case R.id.btItem: {
                    menu.getItem(i).setChecked(PrefsStore.getInstance(this).getBtTransportEnabled());
                    break;
                }
                case R.id.btleItem: {
                    menu.getItem(i).setChecked(PrefsStore.getInstance(this).getBtleTransportEnabled());
                    break;
                }
            }
        }
        return super.onPrepareOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        item.setChecked(!item.isChecked());
        switch (item.getItemId()) {
            case R.id.wifiItem: {
                AIRCONSOLE_WIFI = item.isChecked();
                PrefsStore.getInstance(this).setIpTransportEnabled(AIRCONSOLE_WIFI);
                if (!AIRCONSOLE_WIFI) {
                    mgr.stopScanning(AirconsoleDevice.Transport.TRANSPORT_IP);
                }
                break;
            }
            case R.id.btItem: {
                AIRCONSOLE_BT = item.isChecked();
                PrefsStore.getInstance(this).setBtTransportEnabled(AIRCONSOLE_BT);
                if (!AIRCONSOLE_BT) {
                    mgr.stopScanning(AirconsoleDevice.Transport.TRANSPORT_BT);
                }
                break;
            }
            case R.id.btleItem: {
                AIRCONSOLE_BTLE = item.isChecked();
                PrefsStore.getInstance(this).setBtleTransportEnabled(AIRCONSOLE_BTLE);
                if (!AIRCONSOLE_BTLE) {
                    mgr.stopScanning(AirconsoleDevice.Transport.TRANSPORT_BTLE);
                }
                break;
            }
        }
        updateDeviceList();
        airconsoleScan(true);
        return super.onOptionsItemSelected(item);
    }

    @Override
    public boolean onQueryTextSubmit(String query) {
        return false;
    }

    @Override
    public boolean onQueryTextChange(String query) {
        final List<AirconsoleDevice> filteredAirconsoleList = filter(devices, query);
        adapter = new DiscoveryAdapter(this, filteredAirconsoleList);
        recyclerView.setAdapter(adapter);
        filteringSearchResults = query.length() > 0;
        return true;
    }

    private void loadStoredPrefs() {
        AIRCONSOLE_WIFI = PrefsStore.getInstance(this).getIpTransportEnabled();
        AIRCONSOLE_BT = PrefsStore.getInstance(this).getBtTransportEnabled();
        AIRCONSOLE_BTLE = PrefsStore.getInstance(this).getBtleTransportEnabled();
    }

    private void updateDetectedDevicesLabel() {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                detectedDevicesFooter.setText(String.format("%s %d", getString(R.string.detected_devices_footer), devices.size()));
            }
        });
    }




    private static List<AirconsoleDevice> filter(List<AirconsoleDevice> devices, String query) {
        final String lowerCaseQuery = query.toLowerCase();

        final List<AirconsoleDevice> filteredModelList = new ArrayList<>();
        for (AirconsoleDevice device : devices) {
            final String name = device.getName().toLowerCase();
            final String address = String.valueOf(device.address().toLowerCase());
            if (name.contains(lowerCaseQuery) || address.contains(lowerCaseQuery)) {
                filteredModelList.add(device);
            }
        }
        return filteredModelList;
    }

    private void airconsoleScan(boolean enable) {

        if (enable) {
            if (AIRCONSOLE_WIFI) {
                Log.d(TAG, "Starting Multicast Lock...");
                WifiManager wifi = (WifiManager) this.getApplicationContext().getSystemService(Context.WIFI_SERVICE);
                if (wifi != null) {
                    // get the device ip address
                    final InetAddress deviceIpAddress = getDeviceIpAddress(wifi);
                    multicastLock = wifi.createMulticastLock(getClass().getName());
                    multicastLock.setReferenceCounted(true);
                    multicastLock.acquire();

                    Log.d(TAG, "About to scan IP");
                    HashMap<String, Object> ipParams = new HashMap<>();
                    ipParams.put(AirconsoleMgr.KEY_BINDING_IP, deviceIpAddress);
                    mgr.startScanning(AirconsoleDevice.Transport.TRANSPORT_IP, ipParams);
                    Log.d(TAG, "IP Scan started");
                } else {
                    Log.d(TAG, "Cannot start IP scan as cannot locate WifiManager");
                }
            }
            if (AIRCONSOLE_BT) {
                Log.d(TAG, "About to scan BT");
                HashMap<String, Object> btParams = new HashMap<>();
                btParams.put(AirconsoleMgr.KEY_BLUETOOTH_ADAPTER, BluetoothAdapter.getDefaultAdapter());
                mgr.startScanning(AirconsoleDevice.Transport.TRANSPORT_BT, btParams);
                Log.d(TAG, "BT Scan started");
            }
            if (AIRCONSOLE_BTLE) {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                    if (ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                        requestPermissionsNew();
                    }
                }
                Log.d(TAG, "About to scan BTLE");
                HashMap<String, Object> btleParams = new HashMap<>();
                btleParams.put(AirconsoleMgr.KEY_BLUETOOTH_ADAPTER, BluetoothAdapter.getDefaultAdapter());
                btleParams.put(AirconsoleMgr.KEY_CONTEXT, this.getApplicationContext());
                if (mgr.startScanning(AirconsoleDevice.Transport.TRANSPORT_BTLE, btleParams)) {
                    Log.d(TAG, "BTLE Scan started");
                } else {
                    Log.d(TAG, "BLTE Scan failed");
                }
            }
        } else {
            if (AIRCONSOLE_WIFI) {
                try {
                    mgr.stopScanning(AirconsoleDevice.Transport.TRANSPORT_IP);
                    if (multicastLock != null) {
                        Log.d(TAG, "Releasing Multicast Lock...");
                        multicastLock.release();
                        multicastLock = null;
                    }
                } catch (Exception ex) {
                    Log.e(TAG, ex.getMessage(), ex);
                }
            }
            if (AIRCONSOLE_BT) {
                mgr.stopScanning(AirconsoleDevice.Transport.TRANSPORT_BT);
            }
            if (AIRCONSOLE_BTLE) {
                mgr.stopScanning(AirconsoleDevice.Transport.TRANSPORT_BTLE);
            }
        }
    }

    private InetAddress getDeviceIpAddress(WifiManager wifi) {
        InetAddress result = null;
        try {
            // default to Android localhost
            result = InetAddress.getByName("10.0.0.2");

            // figure out our wifi address, otherwise bail
            WifiInfo wifiinfo = wifi.getConnectionInfo();
            int intaddr = wifiinfo.getIpAddress();
            byte[] byteaddr = new byte[] { (byte) (intaddr & 0xff), (byte) (intaddr >> 8 & 0xff), (byte) (intaddr >> 16 & 0xff), (byte) (intaddr >> 24 & 0xff) };
            result = InetAddress.getByAddress(byteaddr);
        } catch (UnknownHostException ex) {
            Log.d(TAG, String.format("getDeviceIpAddress Error: %s", ex.getMessage()));
        }

        Log.d(TAG, "deviceIP is: "+result);

        return result;
    }

    private void updateDeviceList() {
        if (devices == null) {
            devices = new ArrayList<>();
        }
        if (!devices.contains(mgr.getDefaultDevice())) {
            devices.add(mgr.getDefaultDevice());
        }
        devices.addAll(mgr.getDeviceList());
        if (adapter == null) {
            adapter = new DiscoveryAdapter(this, devices);
            recyclerView.setAdapter(adapter);

        }
        adapter.notifyDataSetChanged();
        updateDetectedDevicesLabel();
    }

    public void deviceAdded(final AirconsoleDevice device) {
        Log.d(TAG, "Device added: " + device.address());
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (!devices.contains(device)) {
                    devices.add(device);
                    adapter.notifyDataSetChanged();
                }
            }
        });
        updateDetectedDevicesLabel();
    }

    public void deviceRemoved(final AirconsoleDevice device) {
        Log.d(TAG, "Device removed: "+ device.address());
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                devices.remove(device);
                adapter.notifyDataSetChanged();
            }
        });
        updateDetectedDevicesLabel();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode,
                                           String permissions[], int[] grantResults) {
        switch (requestCode) {
            case REQUEST_ENABLE_FINE_LOCATION: {
                // If request is cancelled, the result arrays are empty.
                if (grantResults.length > 0
                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    airconsoleScan(true);
                    Snackbar.make(infoLinearLayout, "Location Permission Granted",
                            Snackbar.LENGTH_SHORT)
                            .show();

                } else {
                    Snackbar.make(infoLinearLayout, "Location Permission Denied",
                            Snackbar.LENGTH_SHORT)
                            .show();
                }
            }
        }
    }

    private void displayEnabledBluetoothPrompt() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage("Please enable bluetooth to connect to your Airconsole")
                .setTitle("Your bluetooth is currently disabled!")
                .setCancelable( false )
                .setPositiveButton("OK", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                        startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
                    }
                })
                .setNegativeButton("NO THANKS", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        // Do nothing
                    }
                });
        AlertDialog alert = builder.create();
        alert.show();
    }

    private boolean isBluetoothEnabled() {
        return (BluetoothAdapter.getDefaultAdapter() != null && BluetoothAdapter.getDefaultAdapter().isEnabled());
    }

    private void requestPermissionsNew() {
        if (ActivityCompat.shouldShowRequestPermissionRationale(this, Manifest.permission.ACCESS_FINE_LOCATION)) {
            Snackbar.make(infoLinearLayout, "Location Access Required",
                    Snackbar.LENGTH_INDEFINITE).setAction(android.R.string.ok, new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    // Request the permission
                    ActivityCompat.requestPermissions(DiscoveryActivity.this,
                            new String[]{Manifest.permission.ACCESS_FINE_LOCATION},
                            REQUEST_ENABLE_FINE_LOCATION);
                }
            }).show();
        } else {
            ActivityCompat.requestPermissions(this,
                    new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, 5050);
        }
    }

    private class DiscoveryAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

        Context context;
        private List<AirconsoleDevice> devices;

        DiscoveryAdapter(Context context, List<AirconsoleDevice> devices) {
            this.context = context;
            this.devices = devices;
        }

        @Override
        public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {

            return new ItemViewHolder(LayoutInflater.from(context).inflate(R.layout.list_item_value, parent, false), mListener);
        }

        @Override
        public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
            ItemViewHolder itemViewHolder = (ItemViewHolder) holder;
            AirconsoleDevice device = devices.get(position);
            itemViewHolder.title.setText(device.getName());
            itemViewHolder.value.setText(device.address());
        }

        @Override
        public int getItemCount() {
            return devices.size();
        }
    }
}