package nz.co.cloudstore.airconsoledemo.ui;

import android.content.Intent;
import android.os.AsyncTask;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.util.Pair;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;
import android.widget.TextView;

import java.io.IOException;

import nz.co.cloudstore.airconsole.Airconsole;
import nz.co.cloudstore.airconsole.AirconsoleDevice;
import nz.co.cloudstore.airconsole.AirconsoleSession;
import nz.co.cloudstore.airconsole.AirconsoleSessionListener;
import nz.co.cloudstore.airconsoledemo.DemoAppGlobals;
import nz.co.cloudstore.airconsoledemo.R;

public class SessionActivity extends AppCompatActivity implements AirconsoleSessionListener, AdapterView.OnItemSelectedListener {

    private static final String BAUDRATE_SETTING = "baudrate_setting";
    private static final String DATABITS_SETTING = "databits_setting";
    private static final String PARITY_SETTING = "parity_setting";
    private static final String STOPBITS_SETTING = "stopbits_setting";
    private static final String FLOWCONTROL_SETTING = "flowcontrol_setting";
    private static final String DTR_SETTING = "dtr_setting";
    private static final String RTS_SETTING = "rts_setting";

    private static final String TAG = "SessionActivity";

    private String connectionError;

    private AirconsoleDevice device;
    private AirconsoleSession session;

    private TextView statusTextView, rxBytesTextView, txBytesTextView, modemStatusTextView, signatureTextView, batteryLevelTextView;
    private Spinner baudrateSpinner, databitsSpinner, paritySpinner, stopbitsSpinner, flowcontrolSpinner, dtrSpinner, rtsSpinner;
    private ArrayAdapter<String> baudrateAdapter, databitsAdapter, parityAdapter, stopbitsAdapters, flowcontrolAdapter, dtrAdapter, rtsAdapter;
    private Button sendReceiveTextDataButton;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_session);

        device = DemoAppGlobals.getDevice();
        session = Airconsole.createSession(device);
        session.addSessionListener(this);
        session.connect();


        statusTextView = findViewById(R.id.statusTextView);
        rxBytesTextView = findViewById(R.id.rxBytesTextView);
        txBytesTextView = findViewById(R.id.txBytesTextView);
        modemStatusTextView = findViewById(R.id.modemStatusTextView);
        signatureTextView = findViewById(R.id.signatureTextView);
        batteryLevelTextView = findViewById(R.id.batteryLevelTextView);

        sendReceiveTextDataButton = findViewById(R.id.sendReceiveDataButton);
        sendReceiveTextDataButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent intent = new Intent(SessionActivity.this, TerminalActivity.class);
                startActivity(intent);
            }
        });

        setupSpinners();
    }

    @Override
    public void onItemSelected(AdapterView<?> adapterView, View view, int i, long l) {
        switch (adapterView.getId()) {
            case R.id.baudrateSpinner: {
                processSettingSelected(new Pair(BAUDRATE_SETTING, i));
                break;
            }
            case R.id.databitsSpinner: {
                processSettingSelected(new Pair(DATABITS_SETTING, i));
                break;
            }
            case R.id.paritySpinner: {
                processSettingSelected(new Pair(PARITY_SETTING, i));
                break;
            }
            case R.id.stopbitsSpinner: {
                processSettingSelected(new Pair(STOPBITS_SETTING, i));
                break;
            }
            case R.id.flowcontrolSpinner: {
                processSettingSelected(new Pair(FLOWCONTROL_SETTING, i));
                break;
            }
            case R.id.dtrSpinner: {
                processSettingSelected(new Pair(DTR_SETTING, i));
                break;
            }
            case R.id.rtsSpinner: {
                processSettingSelected(new Pair(RTS_SETTING, i));
                break;
            }
        }
    }

    private void processSettingSelected(Pair settingPair) {
        Log.d(TAG, "processSettingSelected: settingPair=" + settingPair);

        final String key = (String) settingPair.first;
        final int index = (int) settingPair.second;

        switch (key) {
            case BAUDRATE_SETTING: {
                AsyncTask.execute(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            int selectedBaudrate = 0;
                            switch (index) {
                                case 0: {
                                    selectedBaudrate = 2400;
                                    break;
                                }
                                case 1: {
                                    selectedBaudrate = 4800;
                                    break;
                                }
                                case 2: {
                                    selectedBaudrate = 9600;
                                    break;
                                }
                                case 3: {
                                    selectedBaudrate = 19200;
                                    break;
                                }
                                case 4: {
                                    selectedBaudrate = 38400;
                                    break;
                                }
                                case 5: {
                                    selectedBaudrate = 57600;
                                    break;
                                }
                                case 6: {
                                    selectedBaudrate = 115200;
                                    break;
                                }
                            }
                            Log.d(TAG, "processBaudrateSelected: setting baudrate to " + selectedBaudrate);
                            session.setLineParameters(selectedBaudrate, session.getDataBits(), session.getParity(), session.getStopBits());
                        } catch (IOException ioEX) {
                            Log.d(TAG, "io=" + ioEX.toString());
                        }
                    }
                });
                break;
            }
            case DATABITS_SETTING: {
                AsyncTask.execute(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            AirconsoleSession.DataBits selectedDatabit = null;
                            switch (index) {
                                case 0: {
                                    selectedDatabit = AirconsoleSession.DataBits.AC_DATABITS_7;
                                    break;
                                }
                                case 1: {
                                    selectedDatabit = AirconsoleSession.DataBits.AC_DATABITS_8;
                                    break;
                                }
                            }
                            Log.d(TAG, "processDatabitsSelected: setting databits to " + selectedDatabit);
                            session.setLineParameters(session.getBaudRate(), selectedDatabit, session.getParity(), session.getStopBits());
                        } catch (IOException ioEX) {
                            Log.d(TAG, "io=" + ioEX.toString());
                        }
                    }
                });
                break;
            }
            case PARITY_SETTING: {
                AsyncTask.execute(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            AirconsoleSession.Parity selectedParity = null;
                            switch (index) {
                                case 0: {
                                    selectedParity = AirconsoleSession.Parity.AC_PARITY_NONE;
                                    break;
                                }
                                case 1: {
                                    selectedParity = AirconsoleSession.Parity.AC_PARITY_ODD;
                                    break;
                                }
                                case 2: {
                                    selectedParity = AirconsoleSession.Parity.AC_PARITY_EVEN;
                                    break;
                                }
                                case 3: {
                                    selectedParity = AirconsoleSession.Parity.AC_PARITY_MARK;
                                    break;
                                }
                                case 4: {
                                    selectedParity = AirconsoleSession.Parity.AC_PARITY_SPACE;
                                    break;
                                }
                            }
                            Log.d(TAG, "processParitySelected: setting parity to " + selectedParity);
                            session.setLineParameters(session.getBaudRate(), session.getDataBits(), selectedParity, session.getStopBits());
                        } catch (IOException ioEX) {
                            Log.d(TAG, "io=" + ioEX.toString());
                        }
                    }
                });
                break;
            }
            case STOPBITS_SETTING: {
                AsyncTask.execute(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            AirconsoleSession.StopBits selectedStopbit = null;
                            switch (index) {
                                case 0: {
                                    selectedStopbit = AirconsoleSession.StopBits.AC_STOPBITS_1;
                                    break;
                                }
                                case 1: {
                                    selectedStopbit = AirconsoleSession.StopBits.AC_STOPBITS_1_5;
                                    break;
                                }
                                case 2: {
                                    selectedStopbit = AirconsoleSession.StopBits.AC_STOPBITS_2;
                                    break;
                                }
                            }
                            Log.d(TAG, "processStopbitsSelected: setting stopbits to " + selectedStopbit);
                            session.setLineParameters(session.getBaudRate(), session.getDataBits(), session.getParity(), selectedStopbit);
                        } catch (IOException ioEX) {
                            Log.d(TAG, "io=" + ioEX.toString());
                        }
                    }
                });
                break;
            }
            case FLOWCONTROL_SETTING: {
                AsyncTask.execute(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            AirconsoleSession.FlowControl selectedFlowcontrol = null;
                            switch (index) {
                                case 0: {
                                    selectedFlowcontrol = AirconsoleSession.FlowControl.AC_FLOW_NONE;
                                    break;
                                }
                                case 1: {
                                    selectedFlowcontrol = AirconsoleSession.FlowControl.AC_FLOW_SOFTWARE;
                                    break;
                                }
                                case 2: {
                                    selectedFlowcontrol = AirconsoleSession.FlowControl.AC_FLOW_HARDWARE;
                                    break;
                                }
                                case 3: {
                                    selectedFlowcontrol = AirconsoleSession.FlowControl.AC_FLOW_HARDWARE_DSR;
                                    break;
                                }
                            }
                            Log.d(TAG, "processFlowcontrolSelected: setting flowcontrol to " + selectedFlowcontrol);
                            session.setFlowControl(selectedFlowcontrol);
                        } catch (IOException ioEX) {
                            Log.d(TAG, "io=" + ioEX.toString());
                        }
                    }
                });
                break;
            }
            case DTR_SETTING: {
                AsyncTask.execute(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            boolean selectedDtr = index == 0;
                            Log.d(TAG, "processDtrSelected: setting dtr to " + selectedDtr);
                            session.setDTR(selectedDtr);
                        } catch (IOException ioEX) {
                            Log.d(TAG, "io=" + ioEX.toString());
                        }
                    }
                });
                break;
            }
            case RTS_SETTING: {
                AsyncTask.execute(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            boolean selectedRts = index == 0;
                            Log.d(TAG, "processRtsSelected: setting rts to " + selectedRts);
                            session.setDTR(selectedRts);
                        } catch (IOException ioEX) {
                            Log.d(TAG, "io=" + ioEX.toString());
                        }
                    }
                });
                break;
            }
        }

    }

    @Override
    public void onNothingSelected(AdapterView<?> adapterView) {

    }

    private void setupSpinners() {
        baudrateSpinner = findViewById(R.id.baudrateSpinner);
        baudrateAdapter = new ArrayAdapter<String>(this, R.layout.spinner_item, getResources().getStringArray(R.array.baudrate_array));
        baudrateAdapter.setDropDownViewResource(R.layout.spinner_item);
        baudrateSpinner.setAdapter(baudrateAdapter);
        baudrateSpinner.setSelection(3, false);
        baudrateSpinner.setOnItemSelectedListener(this);

        databitsSpinner = findViewById(R.id.databitsSpinner);
        databitsAdapter = new ArrayAdapter<String>(this, R.layout.spinner_item, getResources().getStringArray(R.array.databits_array));
        databitsAdapter.setDropDownViewResource(R.layout.spinner_item);
        databitsSpinner.setAdapter(databitsAdapter);
        databitsSpinner.setSelection(1, false);
        databitsSpinner.setOnItemSelectedListener(this);

        paritySpinner = findViewById(R.id.paritySpinner);
        parityAdapter = new ArrayAdapter<String>(this, R.layout.spinner_item, getResources().getStringArray(R.array.parity_array));
        parityAdapter.setDropDownViewResource(R.layout.spinner_item);
        paritySpinner.setAdapter(parityAdapter);
        paritySpinner.setSelection(0, false);
        paritySpinner.setOnItemSelectedListener(this);

        stopbitsSpinner = findViewById(R.id.stopbitsSpinner);
        stopbitsAdapters = new ArrayAdapter<String>(this, R.layout.spinner_item, getResources().getStringArray(R.array.stopbits_array));
        stopbitsAdapters.setDropDownViewResource(R.layout.spinner_item);
        stopbitsSpinner.setAdapter(stopbitsAdapters);
        stopbitsSpinner.setSelection(0, false);
        stopbitsSpinner.setOnItemSelectedListener(this);

        flowcontrolSpinner = findViewById(R.id.flowcontrolSpinner);
        flowcontrolAdapter = new ArrayAdapter<String>(this, R.layout.spinner_item, getResources().getStringArray(R.array.flowcontrol_array));
        flowcontrolAdapter.setDropDownViewResource(R.layout.spinner_item);
        flowcontrolSpinner.setAdapter(flowcontrolAdapter);
        flowcontrolSpinner.setSelection(0, false);
        flowcontrolSpinner.setOnItemSelectedListener(this);

        dtrSpinner = findViewById(R.id.dtrSpinner);
        dtrAdapter = new ArrayAdapter<String>(this, R.layout.spinner_item, getResources().getStringArray(R.array.dtr_array));
        dtrAdapter.setDropDownViewResource(R.layout.spinner_item);
        dtrSpinner.setAdapter(dtrAdapter);
        dtrSpinner.setSelection(1, false);
        dtrSpinner.setOnItemSelectedListener(this);

        rtsSpinner = findViewById(R.id.rtsSpinner);
        rtsAdapter = new ArrayAdapter<String>(this, R.layout.spinner_item, getResources().getStringArray(R.array.rts_array));
        rtsAdapter.setDropDownViewResource(R.layout.spinner_item);
        rtsSpinner.setAdapter(rtsAdapter);
        rtsSpinner.setSelection(1, false);
        rtsSpinner.setOnItemSelectedListener(this);
    }

    private String formatConnectionStatus() {
        if (session.isConnected()) {
            return "Connected";
        } else if (session.isConnecting()) {
            return "Connecting";
        } else {
            if (connectionError != null) {
                return "Connect failed: " + connectionError;
            } else {
                return "Disconnected";
            }
        }
    }

    private String formatModemStatus() {
        byte msr = session.getMSR();
        String result = "";
        if ((msr & AirconsoleSession.MSR_CTS) > 0) {
            result += getString(R.string.msr_cts) + " ";
        }
        if ((msr & AirconsoleSession.MSR_DSR) > 0) {
            result += getString(R.string.msr_dsr) + " ";
        }
        if ((msr & AirconsoleSession.MSR_RI) > 0) {
            result += getString(R.string.msr_ri) + " ";
        }
        if ((msr & AirconsoleSession.MSR_DCD) > 0) {
            result += getString(R.string.msr_dcd) + " ";
        }
        return result;
    }

    private String formatSignature() {
        String signature = session.getSignature();
        String result = "";
        if (signature != null) {
            result = signature;
        }
        return result;
    }

    private String formatBatteryLevel() {
        int level = session.getBatteryLevel();
        String result;
        if (level  > 0) {
            result = level + "%";
        } else {
            result = "Not available";
        }
        return result;
    }

    private void updateDeviceStatus() {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                statusTextView.setText(formatConnectionStatus());
                rxBytesTextView.setText(Integer.toString(session.getRxByteCount()));
                txBytesTextView.setText(Integer.toString(session.getTxByteCount()));
                modemStatusTextView.setText(formatModemStatus());
                signatureTextView.setText(formatSignature());
                batteryLevelTextView.setText(formatBatteryLevel());

//                if (session.isConnected()) {
//                    // Enable Spinners / Button UI
//                    sendReceiveTextDataButton.setEnabled(true);
//                } else {
//                    // Disable Spinners / Button UI
//                }
            }
        });
    }

    public void sessionWillConnect(AirconsoleSession session) {
        connectionError = null;
        updateDeviceStatus();
    }

    public void sessionDidConnect(AirconsoleSession session) {
        connectionError = null;
        updateDeviceStatus();
    }

    public void sessionFailedToConnect(AirconsoleSession session, String errorMessage) {
        connectionError = errorMessage;
        updateDeviceStatus();
    }

    public void sessionDidDisconnect(AirconsoleSession session) {
        connectionError = null;
        updateDeviceStatus();
    }

    public void sessionDidOverflow(AirconsoleSession session) {
        updateDeviceStatus();
    }

    public void sessionLinePropertiesChanged(AirconsoleSession session) {
        updateDeviceStatus();
    }

    public void sessionModemStatusChanged(AirconsoleSession session) {
        updateDeviceStatus();
    }

    public void sessionSignatureChanged(AirconsoleSession session) {
        updateDeviceStatus();
    }

    public void sessionDidAuthenticate(AirconsoleSession session) {
    }

    public void sessionDidFailToAuthenticate(AirconsoleSession session) {
    }

    public void sessionBatteryLevelChanged(AirconsoleSession session) {
        updateDeviceStatus();
    }
}
