package nz.co.cloudstore.airconsoledemo.ui;

import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;

import nz.co.cloudstore.airconsole.AirconsoleSession;
import nz.co.cloudstore.airconsoledemo.DemoAppGlobals;
import nz.co.cloudstore.airconsoledemo.R;

public class TerminalActivity extends AppCompatActivity {

    private static final String TAG = "TerminalActivity";

    private AirconsoleSession session;

    private Handler textHandler;
    private EditText terminal;
    private RXThread rxThread;
    private OutputStreamWriter writer;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_terminal);

        session = DemoAppGlobals.getSession();

        terminal = (EditText)  findViewById(R.id.et_terminal);
        textHandler = new Handler(Looper.getMainLooper());



        final EditText text = (EditText) findViewById(R.id.et_send);
        final Button button = (Button) findViewById(R.id.btn_send);


        if (session != null) {
            rxThread = new RXThread();
            (new Thread(rxThread)).start();
            try {
                writer = new OutputStreamWriter(session.getOutputStream(), "UTF-8");
            } catch (UnsupportedEncodingException usee) {
                usee.printStackTrace();
                Log.e(TAG, usee.getMessage());
            }

            button.setOnClickListener(new View.OnClickListener() {
                public void onClick(View v) {
                    String s = text.getText().toString() + "\r";
                    text.setText("");
                    try {
                        writer.write(s);
                        writer.flush(); // Call flush to force OutputStreamWriter to write to underlying Airconsole OutputStream
                    } catch (IOException ioe) {
                        ioe.printStackTrace();
                        Log.e(TAG, ioe.getMessage());
                    }
                }
            });
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        // Make sure we clean up the background thread
        Log.d(TAG, "Destroying RX thread");
        if (rxThread != null) {
            rxThread.stop();
            rxThread = null;
        }
    }

    private class RXThread implements Runnable {
        private boolean stopped = false;

        public void stop() {
            stopped = true;
            try {
                if (session != null) {
                    session.flush();
                }
            } catch (IOException ioe) {
                ioe.printStackTrace();
                Log.e(TAG, ioe.getMessage());
            }
        }

        public void run() {
            InputStream is = session.getInputStream();
            try {
                // The input stream will return bytes, we need to convert to characters....
                InputStreamReader isr = new InputStreamReader(is, "UTF-8");

                Log.d(TAG, "RX Thread is running");

                int bufferSize = 5;
                char[] buffer = new char[bufferSize];
                int charsRead;

                do {
                    charsRead = isr.read(buffer, 0, bufferSize);
                    if (charsRead > 0) {
                        // Convert the characters we read into a string
                        final String s = new String(buffer, 0, charsRead);
                        // And append that string to the output text area (need to do on UI/main thread)
                        textHandler.post(new Runnable(){
                            public void run() {
                                // Now on UI thread
                                terminal.append(s);
                            }
                        });
                    }
                } while (!stopped && (charsRead != -1));
            } catch (IOException ioe) {
                ioe.printStackTrace();
                Log.e(TAG, ioe.getMessage());
            }
            Log.d(TAG, "RX Thread is terminating");
        }
    }

}
