package nz.co.cloudstore.airconsole.consoledemo;

import nz.co.cloudstore.airconsole.*;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;

public class Main implements AirconsoleMgrListener, AirconsoleSessionListener {

    private AirconsoleMgr mgr;
    private boolean scanning = true;
    private AirconsoleSession session = null;
    private OutputStream os = null;
    private boolean exitLoop = false;

    public Main(String hostname) {

        System.out.println("Using Airconsole SDK version: "+Airconsole.getSDKVersion());

        mgr = Airconsole.getAirconsoleMgr();
        mgr.addAirconsoleMgrListener(this);

        if (hostname == null) {
            // Start IP based discovery
            HashMap<String, Object> params = new HashMap<String, Object>();
            InetAddress bindingIP = null;
            try {
                //bindingIP = InetAddress.getByName("192.168.10.100");
                bindingIP = InetAddress.getLocalHost();
                System.out.println("binding IP = "+bindingIP);
            } catch (UnknownHostException uhe) {
                uhe.printStackTrace();
            }

            if (bindingIP != null) {
                params.put(AirconsoleMgr.KEY_BINDING_IP, bindingIP);
                if (mgr.startScanning(AirconsoleDevice.Transport.TRANSPORT_IP, params)) {
                    System.out.println("IP scanning started...");
                } else {
                    System.out.println("Failed to start IP scanning - check JmDNS installed");
                }
            } else {
                System.out.println("IP discovery is disabled");
            }
        } else {
            System.out.println("Attempting to connect to host: " + hostname);
            connectToDevice(mgr.getIPDevice(hostname, hostname, -1));
        }
    }

    public void deviceAdded(AirconsoleDevice device) {
        if (scanning) {
            scanning = false;
            System.out.println("Detected Airconsole device "+device.getName()+" at "+device.address());
            connectToDevice(device);
        }
    }

    public void deviceRemoved(AirconsoleDevice device) {}

    public void sessionWillConnect(AirconsoleSession var1) {
        System.out.println("Attempting connection");
    }

    public void sessionDidConnect(final AirconsoleSession session) {
        System.out.println("Session connected");
        try {
            session.setLineParameters(9600, AirconsoleSession.DataBits.AC_DATABITS_8, AirconsoleSession.Parity.AC_PARITY_NONE, AirconsoleSession.StopBits.AC_STOPBITS_1);
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
        os = session.getOutputStream();

        Runnable r = new Runnable() {
            public void run() {
                InputStream is = session.getInputStream();
                while (!exitLoop) {
                    byte[] buffer = new byte[512];

                    try {
                        int bytesRead = is.read(buffer);
                        if (bytesRead > 0) {
                            System.out.write(buffer, 0, bytesRead);
                        } else if (bytesRead == -1) {
                            System.out.println("Stream closed");
                            break;
                        }
                    } catch (IOException ioe) {
                        ioe.printStackTrace();
                        break;
                    }

                }
            }
        };
        (new Thread(r)).start();
    }

    public void sessionFailedToConnect(AirconsoleSession session, String errorMessage) {
        System.out.println("Failed to connect to device: "+errorMessage);
        scanning = true;
    }

    public void sessionDidDisconnect(AirconsoleSession session) {
        System.out.println("Session disconnected");
        exitLoop = true;
    }

    public void sessionDidOverflow(AirconsoleSession session) {
        System.out.println("Session overflow!");
    }

    public void sessionLinePropertiesChanged(AirconsoleSession session) {
        System.out.println("Line settings changed");
    }

    public void sessionModemStatusChanged(AirconsoleSession session) {
        System.out.println("Modem status changed");
    }

    public void sessionSignatureChanged(AirconsoleSession session) {
        System.out.println("Signature has changed to: " + session.getSignature());
    }

    public void sessionDidAuthenticate(AirconsoleSession session) {
        System.out.println("Session authenticated");
    }

    public void sessionDidFailToAuthenticate(AirconsoleSession session) {
        System.out.println("Session failed to authenticate");
    }

    public void sessionBatteryLevelChanged(AirconsoleSession session) {
        System.out.println("Battery level changed to: " + session.getBatteryLevel());
    }

    private void connectToDevice(AirconsoleDevice device) {
        System.out.println("Connecting to device...");
        session = Airconsole.createSession(device);
        exitLoop = false;
        session.addSessionListener(this);
        if (session.connect()) {
        } else {
            System.out.println("Problem connecting to session");
            scanning = true;
        }
    }

    public void inputLoop() {
        InputStream console = System.in;
        while (!exitLoop) {
            byte[] buffer = new byte[512];

            try {
                int bytesRead = console.read(buffer);
                if (bytesRead > 0) {

                    if (os != null) {
                        os.write(buffer, 0, bytesRead);
                    } else {
                        System.out.println("Discarding "+bytesRead+" of input as Airconsole not connected");
                    }

                } else if (bytesRead == -1) {
                    System.out.println("End of input stream");
                    break;
                }
            } catch (IOException ioe) {
                ioe.printStackTrace();
                break;
            }

        }
    }

    public static void main(String[] args) {
        String hostname = null;
        if (args.length > 0) {
            hostname = args[0];
        }
        Main main = new Main(hostname);
        // Start a loop to read from console and send data to Airconsole (if connected)
        main.inputLoop();

        System.out.println("Exiting");
    }
}
